@@ -42,7 +42,7 @@ namespace PerCederberg.Grammatica.Parser {
      * classes.
      *
      * @author   Per Cederberg, <per at percederberg dot net>
-     * @version  1.1
+     * @version  1.2
      */
     public abstract class Node {
 
@@ -202,6 +202,22 @@ namespace PerCederberg.Grammatica.Parser {
             return null; 
         }
     
+        /**
+         * Returns the number of descendant nodes.
+         * 
+         * @return the number of descendant nodes
+         * 
+         * @since 1.2
+         */
+        public int GetDescendantCount() {
+            int  count = 0;
+        
+            for (int i = 0; i < GetChildCount(); i++) {
+                count += 1 + GetChildAt(i).GetDescendantCount();
+            }
+            return count;
+        }
+
         /**
          * Returns the number of computed values associated with this
          * node. Any number of values can be associated with a node
@@ -269,9 +285,11 @@ namespace PerCederberg.Grammatica.Parser {
          * @param values         the vector with node values
          */
         public void AddValues(ArrayList values) {
-            for (int i = 0; i < values.Count; i++) {
-                AddValue(values[i]);
-            }
+        	if (values != null) {
+                for (int i = 0; i < values.Count; i++) {
+                    AddValue(values[i]);
+                }
+        	}
         }
         
         /**
