@@ -47,7 +47,7 @@ import java.util.ArrayList;
  * conflicting sequences can be repeated (would cause infinite loop). 
  *
  * @author   Per Cederberg, <per at percederberg dot net>
- * @version  1.0
+ * @version  1.1
  */
 class LookAheadSet {
 
@@ -184,11 +184,8 @@ class LookAheadSet {
      *  
      * @return true if the next tokens are in the set, or
      *         false otherwise
-     * 
-     * @throws ParseException if the next token couldn't be read 
-     *            or parsed correctly
      */
-    public boolean isNext(Parser parser) throws ParseException {
+    public boolean isNext(Parser parser) {
         Sequence  seq;
 
         for (int i = 0; i < elements.size(); i++) {
@@ -209,13 +206,8 @@ class LookAheadSet {
      *  
      * @return true if the next tokens are in the set, or
      *         false otherwise
-     * 
-     * @throws ParseException if the next token couldn't be read 
-     *            or parsed correctly
      */
-    public boolean isNext(Parser parser, int length) 
-        throws ParseException {
-
+    public boolean isNext(Parser parser, int length) {
         Sequence  seq;
 
         for (int i = 0; i < elements.size(); i++) {
@@ -227,6 +219,49 @@ class LookAheadSet {
         return false;
     }
     
+    /**
+     * Checks if another look-ahead set has an overlapping token 
+     * sequence. An overlapping token sequence is a token sequence
+     * that is identical to another sequence, but for the length. I.e.
+     * one of the two sequences may be longer than the other. 
+     * 
+     * @param set            the look-ahead set to check
+     * 
+     * @return true if there is some token sequence that overlaps, or
+     *         false otherwise
+     */
+    public boolean isOverlap(LookAheadSet set) {
+        for (int i = 0; i < elements.size(); i++) {
+            if (set.isOverlap((Sequence) elements.get(i))) {
+                return true;
+            }
+        }
+        return false;
+    }
+    
+    /**
+     * Checks if a token sequence is overlapping. An overlapping token
+     * sequence is a token sequence that is identical to another 
+     * sequence, but for the length. I.e. one of the two sequences may
+     * be longer than the other. 
+     * 
+     * @param seq            the token sequence to check
+     * 
+     * @return true if there is some token sequence that overlaps, or
+     *         false otherwise
+     */
+    private boolean isOverlap(Sequence seq) {
+        Sequence  elem;
+
+        for (int i = 0; i < elements.size(); i++) {
+            elem = (Sequence) elements.get(i);
+            if (seq.startsWith(elem) || elem.startsWith(seq)) {
+                return true;
+            }
+        }
+        return false;
+    }
+
     /**
      * Checks if the specified token sequence is present in the
      * set.
@@ -321,6 +356,27 @@ class LookAheadSet {
     public void addEmpty() {
         add(new Sequence());
     }
+    
+    /**
+     * Removes a token sequence from this set.
+     * 
+     * @param seq            the token sequence to remove
+     */
+    private void remove(Sequence seq) {
+        elements.remove(seq);
+    }
+
+    /**
+     * Removes all the token sequences from a specified set. Only 
+     * sequences already in this set will be removed.
+     * 
+     * @param set            the set to remove from
+     */
+    public void removeAll(LookAheadSet set) {
+        for (int i = 0; i < set.elements.size(); i++) {
+            remove((Sequence) set.elements.get(i));
+        }
+    }
 
     /**
      * Creates a new look-ahead set that is the result of reading the
@@ -414,6 +470,60 @@ class LookAheadSet {
         return result;
     }
 
+    /**
+     * Creates a new look-ahead set with overlaps from another. All
+     * token sequences in this set that overlaps with the other set
+     * will be added to the new look-ahead set.
+     * 
+     * @param set            the look-ahead set to check with
+     * 
+     * @return a new look-ahead set containing the overlaps
+     */
+    public LookAheadSet createOverlaps(LookAheadSet set) {
+        LookAheadSet  result = new LookAheadSet(maxLength);
+        Sequence      seq;
+        
+        for (int i = 0; i < elements.size(); i++) {
+            seq = (Sequence) elements.get(i);
+            if (set.isOverlap(seq)) {
+                result.add(seq);
+            }
+        }
+        return result;
+    }
+
+    /**
+     * Creates a new look-ahead set filter. The filter will contain
+     * all sequences from this set, possibly left trimmed by each one
+     * of the sequences in the specified set.
+     * 
+     * @param set            the look-ahead set to trim with
+     * 
+     * @return a new look-ahead set filter
+     */
+    public LookAheadSet createFilter(LookAheadSet set) {
+        LookAheadSet  result = new LookAheadSet(maxLength);
+        Sequence      first;
+        Sequence      second;
+        
+        // Handle special cases
+        if (this.size() <= 0 || set.size() <= 0) {
+            return this;
+        }
+
+        // Create combinations
+        for (int i = 0; i < elements.size(); i++) {
+            first = (Sequence) elements.get(i);
+            for (int j = 0; j < set.elements.size(); j++) {
+                second = (Sequence) set.elements.get(j);
+                if (first.startsWith(second)) {
+                    result.add(first.subsequence(second.length()));
+                }
+            }
+        }
+        return result;
+    }
+
     /**
      * Creates a new identical look-ahead set, except for the repeat
      * flag being set in each token sequence.
@@ -441,12 +551,25 @@ class LookAheadSet {
      * @return a string representation of this object
      */
     public String toString() {
+        return toString(null);
+    }
+    
+    /**
+     * Returns a string representation of this object.
+     * 
+     * @param tokenizer      the tokenizer containing the tokens
+     * 
+     * @return a string representation of this object
+     */
+    public String toString(Tokenizer tokenizer) {
         StringBuffer  buffer = new StringBuffer();
+        Sequence      seq;
 
         buffer.append("{");
         for (int i = 0; i < elements.size(); i++) {
+            seq = (Sequence) elements.get(i);
             buffer.append("\n  ");
-            buffer.append(elements.get(i));
+            buffer.append(seq.toString(tokenizer));
         }
         buffer.append("\n}");
         return buffer.toString();
@@ -570,6 +693,28 @@ class LookAheadSet {
             }
         }
 
+        /**
+         * Checks if this token sequence starts with the tokens from
+         * another sequence. If the other sequence is longer than this
+         * sequence, this method will always return false.
+         * 
+         * @param seq            the token sequence to check
+         * 
+         * @return true if this sequence starts with the other, or
+         *         false otherwise
+         */
+        public boolean startsWith(Sequence seq) {
+            if (length() < seq.length()) {
+                return false;
+            }
+            for (int i = 0; i < seq.tokens.size(); i++) {
+                if (!tokens.get(i).equals(seq.tokens.get(i))) {
+                    return false;
+                }
+            }
+            return true;
+        }
+
         /**
          * Checks if this token sequence is repetitive. A repetitive 
          * token sequence is one with the repeat flag set.
@@ -589,11 +734,8 @@ class LookAheadSet {
          * 
          * @return true if the next tokens are in the sequence, or
          *         false otherwise
-         * 
-         * @throws ParseException if the next token couldn't be read 
-         *            or parsed correctly
          */
-        public boolean isNext(Parser parser) throws ParseException {
+        public boolean isNext(Parser parser) {
             Token    token;
             Integer  id;
             
@@ -616,13 +758,8 @@ class LookAheadSet {
          * 
          * @return true if the next tokens are in the sequence, or
          *         false otherwise
-         * 
-         * @throws ParseException if the next token couldn't be read 
-         *            or parsed correctly
          */
-        public boolean isNext(Parser parser, int length) 
-            throws ParseException {
-
+        public boolean isNext(Parser parser, int length) {
             Token    token;
             Integer  id;
             
@@ -645,15 +782,41 @@ class LookAheadSet {
          * @return a string representation of this object
          */
         public String toString() {
+            return toString(null);
+        }
+        
+        /**
+         * Returns a string representation of this object.
+         *
+         * @param tokenizer      the tokenizer containing the tokens
+         *  
+         * @return a string representation of this object
+         */
+        public String toString(Tokenizer tokenizer) {
             StringBuffer  buffer = new StringBuffer();
+            String        str;
+            Integer       id;
 
-            buffer.append(tokens.toString());
+            if (tokenizer == null) {
+                buffer.append(tokens.toString());
+            } else {
+                buffer.append("[");
+                for (int i = 0; i < tokens.size(); i++) {
+                    id = (Integer) tokens.get(i);
+                    str = tokenizer.getPatternDescription(id.intValue());
+                    if (i > 0) {
+                        buffer.append(" ");
+                    }
+                    buffer.append(str);
+                }
+                buffer.append("]");
+            }
             if (repeat) {
                 buffer.append(" *");
             }
             return buffer.toString();
         }
-        
+
         /**
          * Creates a new token sequence that is the concatenation of 
          * this sequence and another. A maximum length for the new 
