@@ -40,7 +40,7 @@ package net.percederberg.grammatica.parser;
  * that must be provided upon creation. 
  *
  * @author   Per Cederberg, <per at percederberg dot net>
- * @version  1.0
+ * @version  1.1
  */
 public class TokenPattern {
 
@@ -255,15 +255,15 @@ public class TokenPattern {
         StringBuffer  buffer = new StringBuffer();
 
         buffer.append(name);
-        buffer.append("(");
+        buffer.append(" (");
         buffer.append(id);
-        buffer.append("): ");
+        buffer.append(") = ");
         if (type == STRING_TYPE) {
-            buffer.append("STRING \"");
+            buffer.append("\"");
             buffer.append(pattern);
             buffer.append("\"");
         } else if (type == REGEXP_TYPE) {
-            buffer.append("REGEXP <<");
+            buffer.append("<<");
             buffer.append(pattern);
             buffer.append(">>");
         }
